class OrderControllerTests extends grails.test.WebFlowTestCase {
    def orderService
    def controller
    def flow

    protected void setUp() {
        controller = new OrderController()
        controller.orderService = orderService
    }

    void testIndex() {
        controller.index()
        assertEquals "/order/search", controller.response.redirectedUrl
    }

    void testGoSearch() {
        controller.goSearch()
        assertEquals "/order/search", controller.response.redirectedUrl
    }

    void testGoCart() {
        controller.goCart()
        assertEquals "/order/cart", controller.response.redirectedUrl
    }

    def getFlow() {
        flow
    }

    def setFlow(flow) {
        this.flow = flow
    }

    // メニューから「検索」
    void testOrderFlowMenuToSearch() {
        setFlow(controller.searchFlow)

        def viewSelection = startFlow()
        assertEquals "search", viewSelection.viewName
    }

    // 検索画面で「検索」
    void testOrderFlowSearchToSearch() {
        setFlow(controller.searchFlow)
        controller.params.title=""
        controller.params.author=""

        def viewSelection = startFlow()
        assertEquals "search", viewSelection.viewName
        viewSelection = signalEvent("doSearch")
        assertEquals "search", viewSelection.viewName
    }

    // 検索画面で「詳細」
    void testOrderFlowSearchToDetail() {
        setFlow(controller.searchFlow)
        controller.params.id=1

        def viewSelection = startFlow()
        assertEquals "search", viewSelection.viewName
        viewSelection = signalEvent("detail")
        assertEquals "displayDetail", viewSelection.viewName
    }

    // 検索画面で「カートに入れる」
    void testOrderFlowSearchToToCart() {
        setFlow(controller.searchFlow)
        controller.params.id=1
        controller.session.order=new Order()

        def viewSelection = startFlow()
        assertEquals "search", viewSelection.viewName
        viewSelection = signalEvent("toCart")
        assertEquals "search", viewSelection.viewName
    }

    // 詳細画面で「検索画面に戻る」
    void testOrderFlowDisplayDetailToSearch() {
        setFlow(controller.searchFlow)
        controller.params.id=1

        def viewSelection = startFlow()
        assertEquals "search", viewSelection.viewName
        viewSelection = signalEvent("detail")
        assertEquals "displayDetail", viewSelection.viewName
        viewSelection = signalEvent("search")
        assertEquals "search", viewSelection.viewName
    }

    // 詳細画面で「カートに入れる」
    void testOrderFlowDisplayDetailToToCart() {
        setFlow(controller.searchFlow)
        controller.params.id=1
        controller.session.order=new Order()

        def viewSelection = startFlow()
        assertEquals "search", viewSelection.viewName
        viewSelection = signalEvent("detail")
        assertEquals "displayDetail", viewSelection.viewName
        viewSelection = signalEvent("toCart")
        assertEquals "displayDetail", viewSelection.viewName
    }

    // メニューから「カートの中を見る」
    void testOrderFlowMenuToCart() {
        setFlow(controller.cartFlow)
        controller.session.order=new Order()

        def viewSelection = startFlow()
        assertEquals "showCart", viewSelection.viewName
    }

    // カートの中を見る画面で「カートの中を編集」
    void testOrderFlowShowCartToEditCart() {
        setFlow(controller.cartFlow)
        controller.session.order=new Order()

        def viewSelection = startFlow()
        assertEquals "showCart", viewSelection.viewName
        viewSelection = signalEvent("editCart")
        assertEquals "editCart", viewSelection.viewName
    }

    // カートの中を見る画面で「精算」
    void testOrderFlowShowCartToCreditCartConfirm() {
        setFlow(controller.cartFlow)
        controller.session.order=new Order()

        def viewSelection = startFlow()
        assertEquals "showCart", viewSelection.viewName
        viewSelection = signalEvent("goCredit")
        assertEquals "showCart", viewSelection.viewName
    }

    // カートの中を編集画面で「編集」
    void testOrderFlowEditCartToSubmit() {
        setFlow(controller.cartFlow)
        controller.session.order=new Order()

        def viewSelection = startFlow()
        assertEquals "showCart", viewSelection.viewName
        viewSelection = signalEvent("editCart")
        assertEquals "editCart", viewSelection.viewName
        viewSelection = signalEvent("submit")
        assertEquals "showCart", viewSelection.viewName
    }

    // カートの中を編集画面で「キャンセル」
    void testOrderFlowEditCartToCancel() {
        setFlow(controller.cartFlow)
        controller.session.order=new Order()

        def viewSelection = startFlow()
        assertEquals "showCart", viewSelection.viewName
        viewSelection = signalEvent("editCart")
        assertEquals "editCart", viewSelection.viewName
        viewSelection = signalEvent("cancel")
        assertEquals "showCart", viewSelection.viewName
    }

    // カート内容確認画面で「次へ」
    void testOrderFlowCreditToNext() {
        setFlow(controller.creditFlow)
        controller.session.order=new Order()

        def viewSelection = startFlow()
        assertEquals "creditCartConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditCustomerConfirm", viewSelection.viewName
    }

    // 支払情報確認画面で「次へ」
    void testOrderFlowCreditAccountConfirmToNext() {
        setFlow(controller.creditFlow)
        controller.session.order=new Order()
        controller.params.name = "my name"
        controller.params.address = "my address"
        controller.params.phoneNumber = "my phoneNumber"
        controller.params.email = "my@email.co.jp"
        controller.params.paymentMethod = "my paymentMethod"

        def viewSelection = startFlow()
        assertEquals "creditCartConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditCustomerConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditFinalConfirm", viewSelection.viewName
    }

    // 支払情報確認画面で「戻る」
    void testOrderFlowCreditAccountConfirmToBack() {
        setFlow(controller.creditFlow)
        controller.session.order=new Order()
        controller.params.name = "my name"
        controller.params.address = "my address"
        controller.params.phoneNumber = "my phoneNumber"
        controller.params.email = "my@email.co.jp"
        controller.params.paymentMethod = "my paymentMethod"

        def viewSelection = startFlow()
        assertEquals "creditCartConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditCustomerConfirm", viewSelection.viewName
        viewSelection = signalEvent("back")
        assertEquals "creditCartConfirm", viewSelection.viewName
    }

    // 最終確認画面で「完了」
    void testOrderFlowCreditFinalConfirmToNext() {
        setFlow(controller.creditFlow)
        controller.session.order=new Order()
        controller.params.name = "my name"
        controller.params.address = "my address"
        controller.params.phoneNumber = "my phoneNumber"
        controller.params.email = "my@email.co.jp"
        controller.params.paymentMethod = "my paymentMethod"

        def viewSelection = startFlow()
        assertEquals "creditCartConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditCustomerConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditFinalConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditComplete", viewSelection.viewName
    }

    // 最終確認画面で「戻る」
    void testOrderFlowCreditFinalConfirmToBack() {
        setFlow(controller.creditFlow)
        controller.session.order=new Order()
        controller.params.name = "my name"
        controller.params.address = "my address"
        controller.params.phoneNumber = "my phoneNumber"
        controller.params.email = "my@email.co.jp"
        controller.params.paymentMethod = "my paymentMethod"

        def viewSelection = startFlow()
        assertEquals "creditCartConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditCustomerConfirm", viewSelection.viewName
        viewSelection = signalEvent("next")
        assertEquals "creditFinalConfirm", viewSelection.viewName
        viewSelection = signalEvent("back")
        assertEquals "creditCustomerConfirm", viewSelection.viewName
    }
}
